local getn, floor, ceil, ipairs, type, insert = getn, floor, ceil, ipairs, type, table.insert;

local function AddItem(self,Name,Key,Refresh,...)
    local Categories = {...}
    local PrevCate = self.Categories
    for i,v in ipairs(Categories) do
        local NewCate = 0 
        for ic,vc in ipairs(PrevCate) do 
            if vc.Name and vc.Name == v then 
                NewCate = ic 
                break
            end 
        end 
        if NewCate == 0 then 
            insert(PrevCate,{Name = v})
            NewCate = getn(PrevCate)
        end
        PrevCate = PrevCate[NewCate]
    end 
    insert(PrevCate,{Name,Key})
    if Refresh then 
        self:Refresh()
    end
end 

local ExpandList = {}
local function SetSelect(self,Type,Key,...)
    local PrevCate = self.Categories 
    wipe(ExpandList)
    if ... then 
        local ToBeExpanded = ""
        for i,v in ipairs({...}) do 
            for ic,vc in ipairs(PrevCate) do 
                if vc.Name and vc.Name == v then 
                    PrevCate = PrevCate[ic]
                    ToBeExpanded = ToBeExpanded .. vc.Name
                    insert(ExpandList,ToBeExpanded)
                    break 
                elseif ic == getn(PrevCate) then 
                    return nil
                end 
            end 
        end 
    end 
    local NewSelect = false 
    if PrevCate[1] and type(PrevCate[1][1]) == "string" then 
        if Type == "Key" then 
            for i,v in ipairs(PrevCate) do 
                if v[2] == Key then 
                    self.SelectedItem = v[2]
                    NewSelect = true 
                    break 
                end 
            end 
        elseif Type == "Name" then 
            for i,v in ipairs(PrevCate) do 
                if v[1] == Key then 
                    self.SelectedItem = v[2]
                    NewSelect = true 
                    break 
                end 
            end 
        elseif Type == "Index" and PrevCate[Key] then 
            self.SelectedItem = PrevCate[Key][2]
            NewSelect = true 
        end 
    end
    if NewSelect then 
        for i,v in ipairs(ExpandList) do 
            self.ExpandedCategories[v] = true 
        end 
    end 
    self:Refresh()
end 

local function BuildCategoryList(self,List, Level, CategoryLink)
    self.SectionSelectBuildIndex = self.SectionSelectBuildIndex or 1
    for i,v in ipairs(List) do 
        if not self["DisplayItem_"..self.SectionSelectBuildIndex] then 
            self["DisplayItem_"..self.SectionSelectBuildIndex] = self.self:CreateButton(self.ScrollFrame.Content,true)
            self["DisplayItem_"..self.SectionSelectBuildIndex]:Size(nil,20)
        end 
        local Height = (self.SectionSelectBuildIndex - 1)*20
        self["DisplayItem_"..self.SectionSelectBuildIndex]:SetPoint("TOPLEFT",(Level - 1)*20,-Height)
        self["DisplayItem_"..self.SectionSelectBuildIndex]:SetPoint("TOPRIGHT",0,-Height)

        local thisCategoryLink = ""

        if List[i][1] and type(List[i][1]) == "string" then 
            self["DisplayItem_"..self.SectionSelectBuildIndex]:SetText(List[i][1], 11, "Normal", nil, nil, "LEFT")
            self["DisplayItem_"..self.SectionSelectBuildIndex].OnClick = function()
                self.SelectedItem = List[i][2]
                if self.OnSelect then 
                    self.OnSelect(List[i][1],List[i][2])
                end 
                self:Refresh()
            end
            self["DisplayItem_"..self.SectionSelectBuildIndex]:SetIcon()
            if self.SelectedItem == List[i][2] then 
                self["DisplayItem_"..self.SectionSelectBuildIndex]:SetFocus(true)
            else 
                self["DisplayItem_"..self.SectionSelectBuildIndex]:SetFocus(false)
            end 
        else 
            thisCategoryLink = CategoryLink .. List[i].Name
            self["DisplayItem_"..self.SectionSelectBuildIndex]:SetText(List[i].Name, 11, "Highlight", nil, nil, "LEFT")
            self["DisplayItem_"..self.SectionSelectBuildIndex].OnClick = function()
                if self.ExpandedCategories[thisCategoryLink] then  
                    self.ExpandedCategories[thisCategoryLink] = nil 
                else 
                    self.ExpandedCategories[thisCategoryLink] = true
                end 
                self:Refresh()
            end
            if self.ExpandedCategories[thisCategoryLink] then 
                self["DisplayItem_"..self.SectionSelectBuildIndex]:SetIcon("TexturePath",self.self:GetIcon("MinusMask"),"Red",1,"Green",0.33,"Blue",0.33,"BorderGap",6,"LockAspectRatio","RIGHT")
            else 
                self["DisplayItem_"..self.SectionSelectBuildIndex]:SetIcon("TexturePath",self.self:GetIcon("PlusMask"),"Red",0.33,"Green",1,"Blue",0.33,"BorderGap",6,"LockAspectRatio","RIGHT")
            end 
            self["DisplayItem_"..self.SectionSelectBuildIndex]:SetFocus(false)
        end 

        self.SectionSelectBuildIndex = self.SectionSelectBuildIndex + 1
        if self.ExpandedCategories[thisCategoryLink] then  
            self:BuildCategoryList(List[i], Level + 1,thisCategoryLink)
        end
    end 
end 

local function Refresh(self,SO)
    self.SectionSelectBuildIndex = 1
    self:BuildCategoryList(self.Categories,1,"")
    self.ScrollFrame.Content:Size(nil,(self.SectionSelectBuildIndex - 1)*20)
    while self["DisplayItem_"..self.SectionSelectBuildIndex] do 
        self["DisplayItem_"..self.SectionSelectBuildIndex]:Release()
        self["DisplayItem_"..self.SectionSelectBuildIndex] = nil 
        self.SectionSelectBuildIndex = self.SectionSelectBuildIndex + 1
    end 
    if SO then 
        self.ScrollFrame:Refresh(SO)
    end 
end 

local function Clear(self,MaintainExpanded)
    self.SectionSelectBuildIndex = 1
    self.SelectedItem = ""
    self.Categories = nil
    self.Categories = {}
    while self["DisplayItem_"..self.SectionSelectBuildIndex] do 
        self["DisplayItem_"..self.SectionSelectBuildIndex]:Release()
        self["DisplayItem_"..self.SectionSelectBuildIndex] = nil 
        self.SectionSelectBuildIndex = self.SectionSelectBuildIndex + 1
    end 
    local SO = self.ScrollFrame.ScrollOffset
    self.ScrollFrame.Content:Size(nil,20)
    if not MaintainExpanded then 
        wipe(self.ExpandedCategories)
    end 
    return SO
end 

function _Avyr_Global_ADDON_Elements_Lib.WidgetsMixin:CreateSectionSelect(Parent)
    local SectionSelect = self:CreateFrame(Parent)

    SectionSelect.ScrollFrame = self:CreateAdancedScrollFrame(SectionSelect)
    SectionSelect.ScrollFrame:SetPoint("TOPLEFT")
    SectionSelect.ScrollFrame.OnContentWidthChanged = function()
    end 

    SectionSelect:SizeChanged(function()
        SectionSelect.ScrollFrame:Size(SectionSelect:GetWidth(),SectionSelect:GetHeight())
    end)

    SectionSelect.Categories = {}
    SectionSelect.ExpandedCategories = {}
    SectionSelect.SelectedItem = ""

    SectionSelect.BuildCategoryList = BuildCategoryList --internal
    SectionSelect.AddItem = AddItem
    SectionSelect.Refresh = Refresh
    SectionSelect.Clear = Clear
    SectionSelect.SetSelect = SetSelect

    return SectionSelect
end 