local getn, floor, ceil, ipairs, type, insert = getn, floor, ceil, ipairs, type, table.insert;

local function Refresh(self, NewIndex, Increment)
    local Refresh = false
    if NewIndex and self.ListOfItems then
        if Increment then
            NewIndex = self.CurrentScroll + NewIndex
        end
        if NewIndex + self.ScrollFrame:GetHeight() > (getn(self.ListOfItems) * self.ItemOffset) + self.self.Padding then
            NewIndex = getn(self.ListOfItems) * self.ItemOffset - self.ScrollFrame:GetHeight() + self.self.Padding
        end
        if NewIndex < 0 then
            NewIndex = 0
        end
        if self.CurrentScroll ~= NewIndex then
            self.CurrentScroll = NewIndex
            Refresh = true
        end
    else
        Refresh = true
    end

    if Refresh then
        if self.ListOfItems then
            local OffsetIndex = floor(self.CurrentScroll / self.ItemOffset)
            if OffsetIndex < 0 then
                OffsetIndex = 0
            end
            if self.CurrentIndexOffset ~= OffsetIndex then
                for i = 1, self.LinesOnDisplay do
                    if self.ListOfItems[OffsetIndex + i] then
                        self.Content["Select_Element_" .. i]:Show()
                        if type(self.ListOfItems[OffsetIndex + i]) == "string" then 
                            self.Content["Select_Element_" .. i]:SetText(self.ListOfItems[OffsetIndex + i])
                            self.Content["Select_Element_" .. i].Tooltip = self.ListOfItems[OffsetIndex + i]
                        else 
                            self.Content["Select_Element_" .. i]:SetText(self.ListOfItems[OffsetIndex + i])
                        end 
                        if self.SelectedIndex == (OffsetIndex + i) then
                            self.Content["Select_Element_" .. i]:SetFocus(true)
                        else
                            self.Content["Select_Element_" .. i]:SetFocus(false)
                        end
                    else
                        self.Content["Select_Element_" .. i]:Hide()
                    end
                    self.CurrentIndexOffset = OffsetIndex
                end
            end
            self.SliderButton:SetPoint("TOP", self.SlidLine, "TOP", 0, -(self.self.Padding + (self.CurrentScroll / self.ItemOffset) * self.ScrollBarHeight / getn(self.ListOfItems)))
            self.ScrollFrame:SetVerticalScroll(self.CurrentScroll % self.ItemOffset)
        else
            for i = 1, self.LinesOnDisplay do
                self.Content["Select_Element_" .. i]:Hide()
            end
            self.SliderButton:SetPoint("TOP", self.SlidLine, "TOP", 0, -(self.self.Padding + self.ScrollBarHeight))
        end
    end
end

local function AddItems(self, Items)
    Items = Items or {}
    self.ListOfItems = Items
    local DisplayedRatio = (self:GetHeight() + self.TopOffset - 2 * self.self.BorderSize) / (getn(self.ListOfItems) * self.ItemOffset + self.self.Padding)
    if DisplayedRatio > 1 then
        DisplayedRatio = 1
    end
    self.SliderButton:SetHeight(self.ScrollBarHeight * DisplayedRatio)
    self.CurrentScroll = 0
    self.CurrentIndexOffset = nil --to force a full reset
    self:Refresh()
end

local function SelectItem(self, Index, UserInput, TextIndex)
    if type(Index) == "string" then
        if self.ListOfItems then 
            for i, v in ipairs(self.ListOfItems) do
                if (type(v) == "table" and v[TextIndex] == Index) or (type(v) == "string" and v == Index) then
                    Index = i;
                    break;
                end
            end
        end 
    end
    if type(Index) == "string" then 
        return --Matching problem
    end 
    if self.SelectedIndex ~= Index then
        if self.SelectedIndex then
            if self.SelectedIndex >= (1 + self.CurrentIndexOffset) and self.SelectedIndex <= (self.CurrentIndexOffset + self.LinesOnDisplay) then
                self.Content["Select_Element_" .. self.SelectedIndex - self.CurrentIndexOffset]:SetFocus(false)
            end
        end
        self.SelectedIndex = Index
        if self.SelectedIndex then
            if self.SelectedIndex >= (1 + self.CurrentIndexOffset) and self.SelectedIndex <= (self.CurrentIndexOffset + self.LinesOnDisplay) then
                self.Content["Select_Element_" .. self.SelectedIndex - self.CurrentIndexOffset]:SetFocus(true)
            end
        end
        if self.OnSelect and UserInput then
            self.OnSelect(self, self.SelectedIndex, self.ListOfItems[self.SelectedIndex])
        end
    end
end

function _Avyr_Global_ADDON_Elements_Lib.WidgetsMixin:CreateSelectList(Parent,NoHeader, ItemSize)
    local Listbox = self:CreateFrame(Parent)

    ItemSize = ItemSize or 20

    Listbox:SetScript("OnMouseWheel",
        function(_, delta)
            Listbox:Refresh(-4 * delta, true)
        end)

    Listbox.DownBtn = self:CreateButton(Listbox)
    Listbox.DownBtn:SetPoint("BOTTOMRIGHT")
    Listbox.DownBtn:Size(20, 20)
    Listbox.DownBtn:SetIcon("TexturePath",self:GetIcon("ArrowHeadMask"),"Theme","Highlight","Rotation",-45,"Zoom",0.2)
    Listbox.DownBtn.OnClick = function(self)
        Listbox:Refresh(Listbox.ItemOffset / 2, true)
    end

    Listbox.TopOffset = -20
    if NoHeader then 
        Listbox.TopOffset = 0
    end 

    Listbox.UpBtn = self:CreateButton(Listbox)
    Listbox.UpBtn:SetPoint("TOPRIGHT", 0, Listbox.TopOffset)
    Listbox.UpBtn:Size(20, 20)
    Listbox.UpBtn:SetIcon("TexturePath",self:GetIcon("ArrowHeadMask"),"Theme","Highlight","Rotation",135,"Zoom",0.2)
    Listbox.UpBtn.Icon:SetTexCoord(-0.2,0.5,0.5,-0.2,0.5, 1.2,1.2,0.5)
    Listbox.UpBtn.OnClick = function(self)
        Listbox:Refresh(-Listbox.ItemOffset / 2, true)
    end

    Listbox.SlidLine = self:FetchTexture(Listbox)
    Listbox.SlidLine:SetPoint("TOP", Listbox.UpBtn, "BOTTOM")
    Listbox.SlidLine:SetPoint("BOTTOM", Listbox.DownBtn, "TOP")
    Listbox.SlidLine:SetWidth(2)
    Listbox.SlidLine:SetColorTexture(1, 0.82, 0)

    Listbox.SliderButton = self:CreateFrame(Listbox)
    Listbox.SliderButton:Size(20, 20)
    Listbox.SliderButton:SetPoint("TOP", Listbox.SlidLine, "TOP", 0, -self.Padding)
    Listbox.SliderButton:SetThemedBackdrop()
    Listbox.SliderButton:SetThemedBackdrop("Backdrop", 1, 0.82, 0)

    Listbox.ScrollFrameBackdrop = self:CreateFrame(Listbox)
    Listbox.ScrollFrameBackdrop:SetThemedBackdrop()
    Listbox.ScrollFrameBackdrop:SetPoint("BOTTOMLEFT")
    Listbox.ScrollFrameBackdrop:SetPoint("TOPRIGHT", Listbox.UpBtn, "TOPLEFT", -self.Padding, 0)

    Listbox.ScrollFrame = self:CreateScrollFrame(Listbox.ScrollFrameBackdrop)
    Listbox.ScrollFrame:SetPoint("BOTTOMLEFT", Listbox.ScrollFrameBackdrop, self.BorderSize, self.BorderSize)
    Listbox.ScrollFrame:SetPoint("TOPRIGHT", Listbox.ScrollFrameBackdrop, -self.BorderSize, -self.BorderSize)

    --Listbox.ScrollFrame:SetThemedBackdrop()
    Listbox.Content = self:CreateFrame(Listbox.ScrollFrame)
    Listbox.Content:SetPoint("TOPLEFT")
    Listbox.Content:Size(100, 100)
    Listbox.Content:SetClampedToScreen(false)
    Listbox.ScrollFrame:SetScrollChild(Listbox.Content)

    Listbox.ItemOffset = 10

    Listbox:SizeChanged(function()
        local MainHeight = Listbox:GetHeight()
        Listbox.LinesOnDisplay = ceil((MainHeight - self.Padding + 20 + Listbox.TopOffset) / (ItemSize + self.Padding))
        --print(Listbox.LinesOnDisplay)
        Listbox.Content:Size(Listbox:GetWidth() - 20 - 2 * self.BorderSize - self.Padding, Listbox.LinesOnDisplay*(ItemSize + self.Padding) + self.Padding)
        Listbox.ItemOffset = 20 + self.Padding
        Listbox.ScrollBarHeight = MainHeight - 60 - 2 * self.Padding + 20 + Listbox.TopOffset
        for i = 1, Listbox.LinesOnDisplay do
            if not Listbox.Content["Select_Element_" .. i] then
                Listbox.Content["Select_Element_" .. i] = self:CreateButton(Listbox.Content)
                insert(Listbox.Content.LM, "Select_Element_" .. i)

                Listbox.Content["Select_Element_" .. i]:SetText("", 11, 1, 1, 1, "LEFT")
                Listbox.Content["Select_Element_" .. i].ActiveCheck = function()
                    return not Listbox.Disable
                end
                Listbox.Content["Select_Element_" .. i].OnClick = function()
                    Listbox:SelectItem(i + floor(Listbox.CurrentScroll / Listbox.ItemOffset), true)
                end
                if Listbox.Content["Select_Element_" .. i] then
                    if i > 1 then
                        Listbox.Content["Select_Element_" .. i]:SetPoint("TOPLEFT",
                            Listbox.Content["Select_Element_" .. i - 1], "BOTTOMLEFT", 0, -self.Padding)
                    else
                        Listbox.Content["Select_Element_" .. i]:SetPoint("TOPLEFT", self.Padding, -self.Padding)
                    end
                end
            end
            Listbox.Content["Select_Element_" .. i]:Size(Listbox.Content:GetWidth() - 2 * self.Padding, ItemSize)
        end
    end)

    Listbox.CurrentScroll = 0

    Listbox.AddItems = AddItems
    Listbox.Refresh = Refresh
    Listbox.SelectItem = SelectItem

    return Listbox
end
